package repositories

//go:generate mkdir -p mocks
//go:generate rm -rf ./mocks/*_minimock.go
//go:generate minimock -i github.com/dmitriygnatenko/internal/interfaces.ArticleTagRepository -o ./mocks/ -s "_minimock.go"

import (
	"context"
	"database/sql"

	sq "github.com/Masterminds/squirrel"
	"github.com/dmitriygnatenko/internal/interfaces"
)

const articleTagTableName = "article_tag"

type articleTagRepository struct {
	db *sql.DB
}

func InitArticleTagRepository(db *sql.DB) interfaces.ArticleTagRepository {
	return articleTagRepository{db: db}
}

func (a articleTagRepository) Add(ctx context.Context, articleID int, tagIDs []int) error {
	if len(tagIDs) == 0 {
		return nil
	}

	qb := sq.Insert(articleTagTableName).
		PlaceholderFormat(sq.Dollar).
		Columns("article_id", "tag_id")

	for _, tagID := range tagIDs {
		qb = qb.Values(articleID, tagID)
	}

	query, args, err := qb.ToSql()
	if err != nil {
		return err
	}

	_, err = a.db.ExecContext(ctx, query, args...)

	return err
}

func (a articleTagRepository) Delete(ctx context.Context, articleID int, tagIDs []int) error {
	if len(tagIDs) == 0 {
		return nil
	}

	query, args, err := sq.Delete(articleTagTableName).
		PlaceholderFormat(sq.Dollar).
		Where(sq.Eq{"article_id": articleID}).
		Where(sq.Eq{"tag_id": tagIDs}).
		ToSql()

	if err != nil {
		return err
	}

	_, err = a.db.ExecContext(ctx, query, args...)

	return err
}

func (a articleTagRepository) DeleteByArticleID(ctx context.Context, articleID int) error {
	query, args, err := sq.Delete(articleTagTableName).
		PlaceholderFormat(sq.Dollar).
		Where(sq.Eq{"article_id": articleID}).
		ToSql()

	if err != nil {
		return err
	}

	_, err = a.db.ExecContext(ctx, query, args...)

	return err
}
